--[[
	Inventory Manager
		By Alexander Brazie

	The Inventory manager is a tool designed to sort and reorder
	everything inside the player's bag either on-demand or upon
	the looting of an enemy corpse. 

	Revision: $Rev$
	Last Author: $Author$
	Last Change: $Date$

  ]]--

-- Global Values
INVENTORY_MANAGER_COLUMNS = 6;
INVENTORY_MANAGER_DEBUG = 0; -- Debug messages on/off
INVENTORY_MANAGER_DEBUG_GUI = 0; -- Debug messages on/off
IM_DEBUG = "INVENTORY_MANAGER_DEBUG"; -- shortcut for me
IM_DEBUG_GUI = "INVENTORY_MANAGER_DEBUG_GUI"; -- shortcut for me

-- Blizzard Registrations
UIPanelWindows["InventoryManagerFrame"] = { area = "left", pushable = 10 };

-- Classification types
CIM_EQUIP		 = 'equip';
CIM_HARVEST		 = 'harvs';
CIM_TRADESKILL	 = 'trade';
CIM_CLOTH		 = 'cloth';
CIM_HERBS		 = 'herbs';
CIM_ORE			 = 'ore  ';
CIM_GEM			 = 'gem  ';
CIM_LEATHER		 = 'leath';
CIM_POTION		 = 'pot  ';
CIM_FISHING		 = 'fishn';
CIM_FIRSTAID	 = 'first';
CIM_ENGINEERING	 = 'engin';
CIM_MISC		 = 'misc ';
CIM_JUNK		 = 'junk ';
CIM_CLASS 		 = 'class';
CIM_AMMO		 = 'ammo ';
CIM_FOOD		 = 'food ';
CIM_DRINK		 = 'drink';
CIM_SHAMAN		 = 'shamn';
CIM_WARLOCK		 = 'wrlck';
CIM_QUESTITEM	 = 'quest';
CIM_LEFTOVER	 = 'lftvr';
CIM_NONE		 = 'none ';

-- Relation between classes and types
CIM_CLASSES = { 
	[CIM_EQUIP]			= { "Weapon", "Armor", "Shield", "Ranged", "Clothing" };
	[CIM_CLOTH]			= { "Thread" };
	[CIM_HERBS]			= { "Herb" };
	[CIM_ORE]			= { "Ore"};
	[CIM_JUNK]			= { "Junk"};
	[CIM_LEATHER]		= { "Leather"};
	[CIM_POTION]		= { "Potion"};
	[CIM_FISHING]		= { "Fishing" };
	[CIM_FIRSTAID]		= { "First Aid" };
	[CIM_ENGINEERING]	= { "Engineering" };
	[CIM_AMMO]			= { "Ammo" };
	[CIM_FOOD]			= { "Food" };
	[CIM_DRINK]			= { "Drink" };
	[CIM_QUESTITEM]		= { "QuestItem"};
	[CIM_MISC]			= { "Misc"};
	[CIM_SHAMAN]		= { "Shaman" };
	[CIM_WARLOCK]		= { "Warlock" };
	[CIM_CLASS]			= { "Warlock", "Mage", "Shaman", "Rogue"};
	[CIM_HARVEST]		= { "Gem", "Herb", "Ore", "Leather" };
	[CIM_TRADESKILL]	= { "First Aid", "Engineering", "Potion", "Fishing" };
	[CIM_LEFTOVER]		= { ".*" };
	[CIM_NONE]			= { "None"}
};

CIM_OPTIONS_TYPE = { 
	[CIM_EQUIP]			= { id=1,  texture ="Interface\\Icons\\Ability_SteelMelee",			text=CIM_EQUIP_TEXT,		label=CIM_EQUIP_LABEL		},
	[CIM_AMMO]			= { id=2,  texture ="Interface\\Icons\\INV_Ammo_Arrow_01",			text=CIM_AMMO_TEXT,			label=CIM_AMMO_LABEL		},
	[CIM_FOOD]			= { id=3,  texture ="Interface\\Icons\\INV_Misc_Food_15",			text=CIM_FOOD_TEXT,			label=CIM_FOOD_LABEL		},
	[CIM_DRINK]			= { id=4,  texture ="Interface\\Icons\\INV_Drink_01",				text=CIM_DRINK_TEXT,		label=CIM_DRINK_LABEL		},
	[CIM_HERBS]			= { id=5,  texture ="Interface\\Icons\\INV_Misc_Herb_12",			text=CIM_HERBS_TEXT,		label=CIM_HERBS_LABEL		},
	[CIM_ORE]			= { id=6,  texture ="Interface\\Icons\\INV_Ore_Copper_01",			text=CIM_ORE_TEXT,			label=CIM_ORE_LABEL			},
	[CIM_POTION]		= { id=7,  texture ="Interface\\Icons\\INV_Potion_03",				text=CIM_POTION_TEXT,		label=CIM_POTION_LABEL		},
	[CIM_CLOTH]			= { id=8,  texture ="Interface\\Icons\\INV_Fabric_Wool_03",			text=CIM_CLOTH_TEXT,		label=CIM_CLOTH_LABEL		},
	[CIM_LEATHER]		= { id=9,  texture ="Interface\\Icons\\INV_Misc_LeatherScrap_08",	text=CIM_LEATHER_TEXT,		label=CIM_LEATHER_LABEL		},
	[CIM_FISHING]		= { id=10, texture ="Interface\\Icons\\INV_Misc_Fish_01",			text=CIM_FISHING_TEXT,		label=CIM_FISHING_LABEL		},
	[CIM_FIRSTAID]		= { id=11, texture ="Interface\\Icons\\INV_Misc_Bandage_02",		text=CIM_FIRSTAID_TEXT,		label=CIM_FIRSTAID_LABEL	},
	[CIM_WARLOCK]		= { id=12, texture ="Interface\\Icons\\Spell_Shadow_SoulGem",		text=CIM_WARLOCK_TEXT,		label=CIM_WARLOCK_LABEL		},
	[CIM_QUESTITEM]		= { id=13, texture ="Interface\\Icons\\INV_Misc_Key_13",			text=CIM_QUESTITEM_TEXT,	label=CIM_QUESTITEM_LABEL	},
	[CIM_MISC]			= { id=14, texture ="Interface\\Icons\\INV_Misc_Coin_06",			text=CIM_MISC_TEXT,			label=CIM_MISC_LABEL		},
	[CIM_CLASS]			= { id=15, texture ="Interface\\Icons\\INV_Misc_Gem_Amethyst_02",	text=CIM_CLASS_TEXT,		label=CIM_CLASS_LABEL		},
	[CIM_HARVEST]		= { id=16, texture ="Interface\\Icons\\INV_Misc_Gem_Stone_01",		text=CIM_HARVEST_TEXT,		label=CIM_HARVEST_LABEL		},
	[CIM_TRADESKILL]	= { id=17, texture ="Interface\\Icons\\INV_Misc_Bandage_01",		text=CIM_TRADESKILL_TEXT,	label=CIM_TRADESKILL_LABEL	},
	[CIM_LEFTOVER]		= { id=18, texture ="Interface\\Icons\\INV_Misc_Gem_Variety_02",	text=CIM_LEFTOVER_TEXT,		label=CIM_LEFTOVER_LABEL	},
	[CIM_ENGINEERING]	= { id=19, texture ="Interface\\Icons\\INV_Misc_Gear_01",			text=CIM_ENGINEERING_TEXT,	label=CIM_ENGINEERING_LABEL	},
	[CIM_JUNK]			= { id=20, texture ="Interface\\Icons\\INV_Misc_TheGoldenCheep",	text=CIM_JUNK_TEXT,			label=CIM_JUNK_LABEL			},
	[CIM_SHAMAN]		= { id=21, texture ="Interface\\Icons\\INV_Banner_01",				text=CIM_SHAMAN_TEXT,		label=CIM_SHAMAN_LABEL		},
	[CIM_NONE]			= { id=22, texture =nil, text=CIM_NONE_TEXT,			label=CIM_NONE_LABEL		}
};
-- Basic Sorting
CIM_S_TYPE		 = "type ";
CIM_S_ALPHA		 = "alpha";
CIM_S_QUALITY	 = "qualt";
CIM_S_ITEMCOUNT	 = "count";
CIM_S_UNIQUE	 = "uniqu";
CIM_S_QUEST		 = "quest";
CIM_S_SOULBOUND	 = "soulb";
CIM_S_ANY		 = "any  ";

-- Reversing
CIM_NORMAL = "0";
CIM_REVERSE = "1";

CIM_OPTIONS_SORTING = {
	[CIM_S_TYPE]		= {	id=1, texture="Interface\\Icons\\INV_Box_03",				text=CIM_S_TYPE_TEXT,			label=CIM_S_TYPE_LABEL		},
	[CIM_S_ALPHA]		= { id=2, texture="Interface\\Icons\\INV_Letter_09",			text=CIM_S_ALPHA_TEXT,			label=CIM_S_ALPHA_LABEL		},
	[CIM_S_QUALITY]		= { id=3, texture="Interface\\Icons\\INV_Misc_OrnateBox",		text=CIM_S_QUALITY_TEXT,		label=CIM_S_QUALITY_LABEL	},
	[CIM_S_ITEMCOUNT]	= { id=4, texture="Interface\\Icons\\INV_Misc_Note_03",			text=CIM_S_ITEMCOUNT_TEXT,		label=CIM_S_ITEMCOUNT_LABEL	},
	[CIM_S_UNIQUE]		= { id=5, texture="Interface\\Icons\\Spell_Frost_WizardMark",	text=CIM_S_UNIQUE_TEXT,			label=CIM_S_UNIQUE_LABEL	},
	[CIM_S_QUEST]		= { id=6, texture="Interface\\Icons\\INV_Misc_Key_13",			text=CIM_S_QUEST_TEXT,			label=CIM_S_QUEST_LABEL		},
	[CIM_S_SOULBOUND]	= { id=7, texture="Interface\\Icons\\INV_Misc_Urn_01",			text=CIM_S_SOULBOUND_TEXT,		label=CIM_S_SOULBOUND_LABEL	},
	[CIM_NONE]			= { id=8, texture=nil,											text=CIM_NONE_TEXT,				label=CIM_NONE_LABEL		}
};

-- Bar Types
CIM_BAR_TYPE="type";
CIM_BAR_SORT="sort";

-- Bar Data
CIM_BARSIZE=7;
CIM_BAROFFSET=0;
CIM_SORTCOUNT = (8);
CIM_TYPECOUNT = (23);
CIM_BARLIST = { CIM_BAR_TYPE, CIM_BAR_SORT };
CIM_BARCOUNTS={ [CIM_BAR_TYPE] = CIM_TYPECOUNT, [CIM_BAR_SORT]=CIM_SORTCOUNT};
CIM_BARNAMES={ IM_TAB1, IM_TAB2 };
CIM_BARTYPE=CIM_BARLIST[1];

-- Databases
CIM_BAGINDEXES = { ["BACK"] = 0, ["BAG1"] = 1, ["BAG2"] = 2, ["BAG3"] = 3, ["BAG4"] = 4 };
CIM_BUTTONIDTOBAGNAME = { [5] = "BACK", [1] = "BAG4", [2] = "BAG3", [3] = "BAG2", [4] = "BAG1" };

-- Order to use bags
InventoryManager_BagOrder = {  "BACK", "BAG1", "BAG2", "BAG3", "BAG4" };
-- Types the bag accepts
InventoryManager_BagTypes = { };
-- Sorting order
InventoryManager_BagSortOrder = { };

-- Save The Mage Hand
RegisterForSave("InventoryManager_BagOrder");
RegisterForSave("InventoryManager_BagTypes");
RegisterForSave("InventoryManager_BagSortOrder");

-- Temporary Bag Order Construct
InventoryManager_BagFutureItems = { }; -- Future slots for bag X stored by index
InventoryManager_BagOverflow = { }; -- Overflow items go here first.
-- Key = Source, Value = Destination
InventoryManager_SwapQueue = { };
InventoryManager_TimeDelta = 0;

-- Used variables
InventoryManager_HeldData = nil;

-- Enables
InventoryManager_Enabled = 0;
InventoryManager_LootEnabled = 0;
InventoryManager_SortStart = 0;
InventoryManager_SortIntoBagOnly = 1;

-- Help String
InventoryManager_Help = { IM_HELP1, IM_HELP2, IM_HELP3, IM_HELP4 }




-- This function handles loading
function InventoryManager_OnLoad() 
	InventoryManager_Initialize();

	this:RegisterEvent("BAG_UPDATE");
	
	-- Tab Handling code
	PanelTemplates_SetNumTabs(this, getn(CIM_BARLIST));
	PanelTemplates_SetTab(this, 1);	
	
	-- Backpack button handling code
	InventoryManager_HookContainerFrameInteraction();
end

function InventoryManager_OnUpdate(time) 
	if ( time ) then
		ChatFrame2:AddMessage( time.." "..InventoryManager_TimeDelta);

		--InventoryManager_TimeDelta = InventoryManager_TimeDelta + time;
	end

	if ( InventoryManager_TimeDelta > 1 ) then 
		--InventoryManager_TimeDelta = 0;	
	end
end


-- This function handle events
function InventoryManager_OnEvent(event) 
	if ( event == "BAG_UPDATE" ) then
		InventoryManager_TrySort();
	end
end

function InventoryManager_TrySort()
		if ( InventoryManager_LootEnabled == 1 or InventoryManager_SortStart == 1 ) then 
			InventoryManager_PrepareForSort();
			InventoryManager_SortAllBags();
			if ( not CursorHasItem() ) then  
				InventoryManager_PerformTopSwapAndPop();					
			else 
			-- Find empty slot and then drop.
				for i=0, 4, 1 do
					--DebugPrint("Dropping: BagCount ("..getn(InventoryManager_BagOrder)..")".."Bag Number: ("..bagnumber..")", IM_DEBUG);
			
					for s=1, GetContainerNumSlots(i) do
						if ( InventoryManager_MakeBagItem(i,s) == nil ) then 
								PickupContainerItem(i,s);
						end
					end
				end
				InventoryManager_PerformTopSwapAndPop();	
			end
		end
		Cosmos_ScheduleByName("IM_SORT", 5, InventoryManager_TrySort);		
end

function InventoryManager_TestOrder()
	-- Order to use bags
	InventoryManager_BagOrder = {  "BAG4", "BAG3", "BAG2", "BAG1", "BACK" };
	
	InventoryManager_BagTypes[CIM_BAGINDEXES["BAG4"]] = {
		{CIM_HERBS, CIM_NORMAL}
	};
	InventoryManager_BagSortOrder[CIM_BAGINDEXES["BAG4"]] = {
		{CIM_S_TYPE, CIM_REVERSE}, {CIM_S_ALPHA, CIM_NORMAL}, {CIM_S_ITEMCOUNT, CIM_NORMAL}
	};
	InventoryManager_BagTypes[CIM_BAGINDEXES["BAG3"]] = {
		{CIM_POTION, CIM_NORMAL}, {CIM_TRADESKILL, CIM_NORMAL}
	};
	InventoryManager_BagSortOrder[CIM_BAGINDEXES["BAG3"]] = {
		{CIM_S_TYPE, CIM_NORMAL}, {CIM_S_ALPHA, CIM_NORMAL}, {CIM_S_ITEMCOUNT, CIM_NORMAL}
	};
	InventoryManager_BagTypes[CIM_BAGINDEXES["BAG2"]] = {
		{CIM_CLASS, CIM_NORMAL}, {CIM_QUESTITEM, CIM_NORMAL}
	};
	InventoryManager_BagSortOrder[CIM_BAGINDEXES["BAG2"]] = {
		{CIM_S_TYPE, CIM_NORMAL}, {CIM_S_ALPHA, CIM_REVERSE}, {CIM_S_ITEMCOUNT, CIM_NORMAL}
	};
	InventoryManager_BagTypes[CIM_BAGINDEXES["BAG1"]] = {
		{CIM_EQUIP, CIM_NORMAL}
	};
	InventoryManager_BagSortOrder[CIM_BAGINDEXES["BAG1"]] = {
		{CIM_S_TYPE, CIM_NORMAL}, {CIM_S_ALPHA, CIM_NORMAL}, {CIM_S_ITEMCOUNT, CIM_NORMAL}
	};
	InventoryManager_BagTypes[CIM_BAGINDEXES["BACK"]] = {
		{CIM_HARVEST, CIM_NORMAL}, {CIM_LEFTOVER, CIM_NORMAL}
	};
	InventoryManager_BagSortOrder[CIM_BAGINDEXES["BACK"]] = {
		{CIM_S_QUALITY, CIM_NORMAL}, {CIM_S_TYPE, CIM_NORMAL}, {CIM_S_ALPHA, CIM_NORMAL}, {CIM_S_ITEMCOUNT, CIM_NORMAL}
	};
end


-- This function on show
function InventoryManager_OnShow()
--	InventoryManager_LoadVariables();
	InventoryManagerIndexRefresh();
	InventoryManagerTypeRefresh();
end

-- This function is for clearing the GUI 
function InventoryManager_OnHide()
	InventoryManager_SaveVariables();
end

-- This function is for initializing the database
function InventoryManager_Initialize()
	InventoryManager_Reset();
	--InventoryManager_LoadVariables();

	Cosmos_RegisterButton ( 
		IM_BTN, 
		IM_BTN_SUB, 
		IM_BTN_TIP, 
		"Interface\\Icons\\INV_Gauntlets_16", 
		ToggleMageHand
		);
		
	-- Register the Cosmos Menu Options
	Cosmos_RegisterConfiguration(
		"COS_INVM",
		"SECTION",
		IM_SEP,
		IM_SEP_INFO
		);
	Cosmos_RegisterConfiguration(
		"COS_INVM_SEPARATOR",
		"SEPARATOR",
		IM_SEP,
		IM_SEP_INFO
		);
	--[[
	Cosmos_RegisterConfiguration
		"COS_INVM_SHOWTOGGLE",
		"BUTTON",
		"Show the Mage Hand Interface",
		"Click here to show the Mage Hand window.",
		ToggleMageHand()
		);
	]]
	Cosmos_RegisterCVar("COS_INVM_ENABLE_X","0");
	Cosmos_SetCVar("COS_INVM_ENABLE_X","0");
	Cosmos_RegisterConfiguration(
		"COS_INVM_ENABLE",
		"CHECKBOX",
		IM_CHECK_ENABLE,
		IM_CHECK_ENABLE_INFO,
		InventoryManager_EnableToggle,
		1
		);
	Cosmos_RegisterCVar("COS_INVM_LOOTTOGGLE_X","0");
	Cosmos_SetCVar("COS_INVM_LOOTTOGGLE_X","0");
	Cosmos_RegisterConfiguration(
		"COS_INVM_LOOTTOGGLE",
		"CHECKBOX",
		IM_CHECK_AUTOSORT,
		IM_CHECK_AUTOSORT_INFO,
		InventoryManager_LootToggle,
		0
		);

	--Create a function for your command:
	local func = function() ShowUIPanel(InventoryManagerFrame); end
	
	--Create a list of /commands you want applied. 
	local cmdlist = IM_COMM;
	
	--Create a help description 
	local desc = IM_DESC;
	
	--Pick a name or an action you are overwriting.
	local funcname = "MAGEHANDOPEN";
	
	--Choose how to handle a command if you overwrite it. (usually CSM_CHAINNONE)
	local chain = CSM_CHAINNONE; 
	
	--Register it
	Cosmos_RegisterChatCommand( funcname, cmdlist, func, desc, chain );
end

-- Helper Functions

function ToggleMageHand()
	if ( InventoryManagerFrame:IsVisible() ) then 
		HideUIPanel(InventoryManagerFrame);
	else 
		ShowUIPanel(InventoryManagerFrame);
	end
end

-- This function toggles the ability to sort on/off
function InventoryManager_EnableToggle(toggle) 
	InventoryManager_Enabled = toggle;
	InventoryManager_FixBackPackButtons();
end

-- This function toggles the ability to sort on/off
function InventoryManager_LootToggle(toggle) 
	InventoryManager_LootEnabled = toggle;
end

-- This function resets the bag sorting boxes
function InventoryManager_Reset()
	InventoryManager_BagOverflow = { };
	InventoryManager_SwapQueue = { };
	
	-- Set the bag sorting to empty.
	for k,v in CIM_BAGINDEXES do
		InventoryManager_BagTypes[v] = { };
		InventoryManager_BagSortOrder[v] = { };
		InventoryManager_BagFutureItems[v] = { };
		for i = 0, INVENTORY_MANAGER_COLUMNS, 1 do 
			-- Register the bag types
			if ( k == 0 ) then
				if ( i == 1 ) then 
					InventoryManager_BagTypes[v][i] = {CIM_FOOD, CIM_NORMAL};
				else
					InventoryManager_BagTypes[v][i] = {CIM_NONE, CIM_NORMAL};			
				end
			else
				if ( i == 1 ) then 
					InventoryManager_BagTypes[v][i] = {CIM_MISC, CIM_NORMAL};
				else
					InventoryManager_BagTypes[v][i] = {CIM_NONE, CIM_NORMAL};			
				end
			end
			
			-- Register the bag sort order
			if ( i == 1 ) then 
				InventoryManager_BagSortOrder[v][i] = {CIM_S_TYPE, CIM_NORMAL};
			elseif ( i == 2 ) then
				InventoryManager_BagSortOrder[v][i] = {CIM_S_ALPHA, CIM_NORMAL};
			elseif ( i == 3 ) then
				InventoryManager_BagSortOrder[v][i] = {CIM_S_ITEMCOUNT, CIM_NORMAL};
			else
				InventoryManager_BagSortOrder[v][i] = {CIM_NONE, CIM_NORMAL};
			end
		end

	end
end

-- Prepares for sorting by clearing out the arrays
function InventoryManager_PrepareForSort() 
	InventoryManager_BagOverflow = { };
	InventoryManager_SwapQueue = { };

	for k,v in CIM_BAGINDEXES do
		InventoryManager_BagFutureItems[v] = { };
	end
end

-- Load the stored sort order
function InventoryManager_LoadVariables()
	-- Yay for RegisterForSave
end

-- Save the stored sort order
function InventoryManager_SaveVariables()
	-- Automatically done
end

-- Save the stored sort order
function imprint()
	-- Set the bag sorting to empty.
	for k,v in CIM_BAGINDEXES do
		local out = "Bag Index ("..k..")";
		for i = 0, INVENTORY_MANAGER_COLUMNS, 1 do 
			out = out.." "..i..": ";
			if ( InventoryManager_BagTypes[v][i] ) then
				out = out .." t1 "..InventoryManager_BagTypes[v][i][1];
				out = out .." t2 "..InventoryManager_BagTypes[v][i][2];
			end
			if ( InventoryManager_BagSortOrder[v][i] ) then
				out = out.." Sort: ";
				out = out.." s1-"..InventoryManager_BagSortOrder[v][i][1];
				out = out.." s2-"..InventoryManager_BagSortOrder[v][i][2];
			end
		end
		DebugPrint(out, IM_DEBUG);
	end
end

--[[
-- Inventory Functions
-- ]]
function imds() 
	InventoryManager_Reset();
	--InventoryManager_TestOrder();
	--InventoryManager_PrepareForSort();

	InventoryManager_SortAllBags();
	PrintFutureBagState();
	--InventoryManager_PrintSwapList();
end

function SortAll()
	InventoryManager_SortStart = 1;
	InventoryManager_PrepareForSort();

	InventoryManager_SortAllBags();
	--PrintFutureBagState();
	--InventoryManager_PrintSwapList();
end
-- Sort all
function InventoryManager_SortAllBags() 
	-- Iterate over every item
	local n = getn(InventoryManager_BagOrder);
	for i,v in InventoryManager_BagOrder do
		local bagnumber = CIM_BAGINDEXES[InventoryManager_BagOrder[i]]; -- Real bag number
		DebugPrint("Checking Bags in order...("..i..")", IM_DEBUG);
		DebugPrint(InventoryManager_BagOrder[i].." is "..bagnumber, IM_DEBUG);
		
		for slot=1, GetContainerNumSlots(bagnumber),1 do
			InventoryManager_InsertOrdered(bagnumber, slot);		
		end
	end
 	EmptyOverflowIntoBags();
 	InventoryManager_CreateSwapList();
	if ( table.getn(InventoryManager_SwapQueue) == 0 ) then 
		InventoryManager_SortStart = 0;
	end
end
-- If there is a queue that exists, do it!
function InventoryManager_PerformTopSwapAndPop() 
	if ( table.getn( InventoryManager_SwapQueue ) > 0 ) then 
		if(CursorHasItem()) then
			ErrorPrint("It looks like you're already holding something.\nDrop the item to continue sorting.");
		else
			swap = table.remove( InventoryManager_SwapQueue );
			--Print("Swapping from ("..swap[1]..","..swap[2]..") -> ("..swap[3]..","..swap[4]..")");
			InventoryManager_LastDestination = { swap[3],swap[4],-1,-1 };
			PickupContainerItem(swap[1],swap[2]);
			PickupContainerItem(swap[3],swap[4]);
		end
	end
end	

-- Makes the bag item
function InventoryManager_MakeBagItem(bag,slot)
	local classification, name, quality, itemCount, leftString, rightString, minLevel, unique, soulbound, bindsOnPickup, bindsOnEquip, questItem = ClassifyItem(bag,slot, "CosmosTooltip");

	if ( name == "" ) then DebugPrint("No item in bag "..bag.." slot: "..slot, IM_DEBUG); return; end
	if(not itemCount) then
	  return;
	end

	DebugPrint("Item is "..name.." of type: "..classification..".", IM_DEBUG);

	bagitem = { slot = 0; class = classification; name = name; quality = quality; count = itemCount; unique = unique; soulbound = soulbound; bindp = bindsOnPickup, binde = BindsOnEquip; questItem = questItem; oldbag = bag; oldslot = slot }; 	
	return bagitem;
end	

-- Insert a bag/slot item in order. 
function InventoryManager_InsertOrdered(bag, slot)
	--DebugPrint("Applying bagorder to Bag# "..bag.." Slot "..slot, IM_DEBUG);
	bagitem = InventoryManager_MakeBagItem(bag, slot);
	if ( bagitem ) then 
		InventoryManager_InsertSorted(bagitem, 1);
	end
end

function InventoryManager_InsertSorted(bagitem, startbagindex) 
	if ( bagitem == nil ) then return; end

	local bagmatch = -1;

	DebugPrint("Starting Bag: ("..startbagindex..") ", IM_DEBUG);
	-- Check each bag in order
	for i=startbagindex,getn(InventoryManager_BagOrder),1 do
		DebugPrint("Checking Bags in order... #"..i.." is bag #"..CIM_BAGINDEXES[InventoryManager_BagOrder[i]], IM_DEBUG);
		local bagnumber = CIM_BAGINDEXES[InventoryManager_BagOrder[i]]; -- Real bag number
		local bagorder = InventoryManager_BagSortOrder[bagnumber];
		local bagtype = InventoryManager_BagTypes[bagnumber];		
		local currentbag = InventoryManager_BagFutureItems[bagnumber];
						
		-- each typelist contains an array of classes
		for key, typename in bagtype do
			-- If any item in the class list is in the item's class
			--DebugPrint("bagtype key"..key.." k:"..typename[1], IM_DEBUG);
				
			if ( IsWordInClassList(typename[1], bagitem.class ) ) then
				DebugPrint("This bag "..InventoryManager_BagOrder[i].."("..i..") contains item ["..bagitem.name.."]'s matching class: "..bagitem.class, IM_DEBUG);
				-- We've found a bag candidate
				bagmatch = bagnumber;	
			end
		end

		-- If the bagmatch matches..
		if (bagmatch ~= -1) then
			InsertIntoListSorted(InventoryManager_BagFutureItems[bagnumber], bagitem, CompareUsingOrderList( bagorder, bagtype ) );					
			DebugPrint("Item was added in InsertSorted", IM_DEBUG)
		
			if ( getn(currentbag) > GetContainerNumSlots(bagnumber) ) then
				DebugPrint("This bag is full, adding and then handling overflow.", IM_DEBUG);
				HandleOverflow(bagnumber, GetContainerNumSlots(bagnumber), i );
				return;
			else
				DebugPrint("InventoryManager Inserting into bag");
				return;
			end				
			this:RegisterForClicks("LeftButtonDown", "LeftButtonUp");
		end
	end

	if ( bagmatch == -1 ) then 
		DebugPrint("No bag matching the item class was found. Adding to overflow bag.", IM_DEBUG);
		AddToOverflowBag(bagitem);
	end
end

-- Adds to the overflow bag
function AddToOverflowBag (bagitem)
	DebugPrint("Setting an item("..bagitem.name..") aside for overflow handling later", IM_DEBUG);

	InsertIntoListSorted(InventoryManager_BagOverflow, bagitem, CompareUsingOrderList( {{CIM_S_QUALITY, CIM_NORMAL},{CIM_S_TYPE, CIM_NORMAL}, {CIM_S_ALPHA, CIM_NORMAL}}, {{CIM_LEFTOVER, CIM_NORMAL}} ) );
	DebugPrint("Item was added in AddtoOverflowBag", IM_DEBUG);
end

function DropIntoLeftoverBag( bagitem ) 
	for i=1, getn(InventoryManager_BagOrder), 1 do
		local bagnumber = CIM_BAGINDEXES[InventoryManager_BagOrder[i]]; -- Real bag number
		local bagorder = InventoryManager_BagSortOrder[bagnumber];
		local bagtype = InventoryManager_BagTypes[bagnumber];		
		local currentbag = InventoryManager_BagFutureItems[bagnumber];
		
		if ( InventoryManager_GetClassIndex(bagtype, CIM_LEFTOVER) < 7 ) then 
			if ( getn(currentbag) >= GetContainerNumSlots(bagnumber) ) then
				DebugPrint("This leftover bag is full. Adding to first available location", IM_DEBUG);
				return DropIntoFirstAvailableSpot( bagitem );
			else
				DebugPrint("InventoryManager is inserting into leftovers bag", IM_DEBUG);
				if ( InventoryManager_SortIntoBagOnly == 1 ) then 
					InsertIntoListUnSorted(InventoryManager_BagFutureItems[bagnumber], bagitem, CompareUsingOrderList( bagorder, bagtype ) );
				else
					InsertIntoListSorted(InventoryManager_BagFutureItems[bagnumber], bagitem, CompareUsingOrderList( bagorder, bagtype ) );
				end
				DebugPrint("Item was added in DropLeftoverBag", IM_DEBUG);
				return true;
			end
		end
	end

	DebugPrint("No leftover bag found. Adding to first available location.", IM_DEBUG);
	return DropIntoFirstAvailableSpot( bagitem );

end

-- Inserts into first unoccupied slot
function DropIntoFirstAvailableSpot( bagitem ) 
	local wasInserted = false;
	local start, endi, inc = 1,1,1;
	DebugPrint("Dropping "..bagitem.name.." into first available slot.", IM_DEBUG);

	if ( true ) then 
		start, endi, inc = getn(InventoryManager_BagOrder), 1, -1;
	else
		start, endi, inc = 1,getn(InventoryManager_BagOrder), 1;
	end
	
	for i=start, endi, inc do
		local bagnumber = CIM_BAGINDEXES[InventoryManager_BagOrder[i]]; -- Real bag number
		local bagorder = InventoryManager_BagSortOrder[bagnumber];
		local bagtype = InventoryManager_BagTypes[bagnumber];		
		local currentbag = InventoryManager_BagFutureItems[bagnumber];

		--DebugPrint("Dropping: BagCount ("..getn(InventoryManager_BagOrder)..")".."Bag Number: ("..bagnumber..")", IM_DEBUG);
		
		if ( getn(currentbag) < GetContainerNumSlots(bagnumber) ) then 
			DebugPrint("Bag #"..bagnumber.." has an empty slot. ", IM_DEBUG);
			
			if ( true ) then 
				AddToListBackwards(currentbag,bagitem,GetContainerNumSlots(bagnumber));
				DebugPrint("Item was added backwards in DropIntoFirstAvailableSlot", IM_DEBUG);
			else
				if ( InventoryManager_SortIntoBagOnly == 1 ) then 
					InsertIntoListUnSorted(currentbag, bagitem, CompareUsingOrderList( bagorder, bagtype ) );
				else
					InsertIntoListSorted(currentbag, bagitem, CompareUsingOrderList( bagorder, bagtype ) );
				end
				DebugPrint("Item was added in DropIntoFirstAvailableSlot", IM_DEBUG);
			end

			wasInserted = true;
			return wasInserted;
		end
	end

	return wasInserted;
end

-- Handle bag overflow (Messy looking, simple concept)
function HandleOverflow(bagnumber, baglimit, startbagindex)
	DebugPrint("Beginning overflow handling on bag "..bagnumber..". Bag limit is "..baglimit, IM_DEBUG);
	
	for k,v in InventoryManager_BagFutureItems[bagnumber] do
		DebugPrint("Bag Contents: " ..k.." "..v.name, IM_DEBUG);
	end
	
	while ( getn(InventoryManager_BagFutureItems[bagnumber]) > baglimit ) do 
		-- Remove the overflow item.
		local bagitem = table.remove(InventoryManager_BagFutureItems[bagnumber],baglimit+1);
		DebugPrint("Overflow item is ".. bagitem.name, IM_DEBUG);
				
		if ( startbagindex < getn(InventoryManager_BagOrder) ) then 
			-- Insert it into the next bag first
			DebugPrint("Attempting to drop into next bag first: "..startbagindex+1, IM_DEBUG);
			InventoryManager_InsertSorted(bagitem, startbagindex+1);
		else 
			DebugPrint("Dropping item into overflow bag", IM_DEBUG);		
			-- Drop the item into another bag
			AddToOverflowBag(bagitem);
		end
	end
end

-- Empty the overflow pile into the bags
function EmptyOverflowIntoBags()
	local bagitem = table.remove(InventoryManager_BagOverflow) 
		--[[
	while ( bagitem ~= nil) do
		if ( DropIntoLeftoverBag(bagitem) ) then
			DebugPrint("Item Successfulled removed from overflow: "..bagitem.name.." ("..getn(InventoryManager_BagOverflow)..") remaining.", IM_DEBUG);
		else 
			Print("Overflow Error. Items may not be completely sorted.");
		end
		bagitem = table.remove(InventoryManager_BagOverflow);
	end
	]]
end		

-- Print the future location of all items in all bags.
function PrintFutureBagState () 
	for k,v in InventoryManager_BagFutureItems do 
		for k2,item in v do
			Print(" ".. GetIndexInList(CIM_BAGINDEXES,k).." "..k2.." : "..item.name.." x"..item.count.." ["..item.class.."]".." ("..item.oldbag..","..item.oldslot..")",nil,nil,nil, ChatFrame3);
		end
	end
end

-- Prints the list of swaps!
function InventoryManager_PrintSwapList()
	if ( getn( InventoryManager_SwapQueue ) == 0 ) then 
		Print ( "No Swaps Nessecary." );
	end
	for k,v in InventoryManager_SwapQueue do
		Print ( " ("..v[1]..","..v[2]..") -> ("..v[3]..","..v[4]..")" );
	end
end

--- Create the swaplist!
function InventoryManager_CreateSwapList() 
	for k,v in InventoryManager_BagFutureItems do 
		for k2,item in v do
			if ( k == item.oldbag and k2 == item.oldslot ) then
				--Print(item.name.." is located correctly."); 
			elseif ( InventoryManager_BagFutureItems[item.oldbag][item.oldslot] ) then
				if ( item.name == InventoryManager_BagFutureItems[item.oldbag][item.oldslot].name and item.count == InventoryManager_BagFutureItems[item.oldbag][item.oldslot].count) then
					DebugPrint(item.name.." was almost swapped with an equal item.", IM_DEBUG);
				else
					table.insert(InventoryManager_SwapQueue, {item.oldbag,item.oldslot,k,k2} );
				end
			else
				table.insert(InventoryManager_SwapQueue, {item.oldbag,item.oldslot,k,k2} );
			end
			--table.remove(InventoryManager_BagFutureItems[k], k2);
			InventoryManager_UpdateOldBagSlot(k,k2, item.oldbag, item.oldslot)

			--DebugPrint("Finished: "..k.." "..k2.." : "..item.name.." x"..item.count.." ["..item.class.."]".." ("..item.oldbag..","..item.oldslot..")", IM_DEBUG );
		end
	end
end
-- Update item's old x/y  with another items old x/y
function InventoryManager_UpdateOldBagSlot(oldbag, oldslot, newoldbag, newoldslot ) 
	for k,v in InventoryManager_BagFutureItems do 
		if ( k < oldbag) then
		else
			for k2,item in v do
				if ( k == oldbag and k2 <= oldslot ) then 
				else 
					if ( item.oldbag == oldbag  and item.oldslot == oldslot ) then
						--DebugPrint("Swapping old values: "..oldbag..","..oldslot..' with '..newoldbag..","..newoldslot, IM_DEBUG);
						item.oldbag = newoldbag;
						item.oldslot = newoldslot;
					end
				end
			end
		end
	end
end

-- Tooltip setter
function InventoryManager_SetItemToolTip(buttonframe) 

end

-- ------------- Handy Helper Functions --------------------

-- Compare should return 1 when a > b
function InsertIntoListSorted(list, item, compare) 
	for i=1,getn(list),1 do 
		if ( compare(list[i],item) == 1)  then
			DebugPrint("#LS#1 Item "..item.name.." added at "..i, IM_DEBUG);
			table.insert(list,i,item);
			return;
		end
	end
	DebugPrint("#LS#2 Item "..item.name.." added at "..(getn(list)+1), IM_DEBUG);
	table.insert(list,item);
end

-- Compare should return 1 when a > b
function InsertIntoListUnSorted(list, item) 
	DebugPrint("#LS#2 Item "..item.name.." added at "..(getn(list)+1), IM_DEBUG);
	table.insert(list,item);
end

function GetListIndexIfInSubList (list, item) 
	for k,sublist in list do
		for k2,v in sublist do 
			if ( v == item ) then return k; end
		end
	end
	return getn(list)+1;
end


function IsItemInSublist (list, item) 
	for k,sublist in list do
		for k2,v in sublist do 
			if ( v == item ) then return true; end
		end
	end
	return false;
end

function InventoryManager_GetClassIndex (typelist, class)
	if ( typelist ~= nil) then 
		for k,v in typelist do
			--DebugPrint(k.." "..v[1],IM_DEBUG);
			if (CIM_CLASSES[v[1]]) then
				if ( IsListItemInWord(CIM_CLASSES[v[1]], class ) ) then
					return k;
				end
			end
		end
	else
		DebugPrint("Nil typelist supplied.", IM_DEBUG);
		return; 
	end
	return getn(typelist) + 1;
end

-- Checks if any item from a list is in a specified word

function IsWordInClassList(class, word) 
	for k,v in CIM_CLASSES[class] do
		if ( string.find(word,v) ~= nil ) then
			return true;
		end
	end
	return false;
end

function AddToListBackwards(list,item,index)
	for i=index,1,-1 do 
		if ( list[i] == nil ) then 
			DebugPrint("#BK#1 Item "..item.name.." added at "..i, IM_DEBUG);
			
			list[i] = item;
			return;
		end
	end
	DebugPrint("#BK#2 Item "..item.name.." added at "..(getn(list)+1), IM_DEBUG);
	table.insert(list,item);
end

-- This function compares two item objects using an ordering list
function CompareUsingOrderList(orderlist, typelist) 
	-- Generates an expanded list for searching
	return 
	function (a, b)
		for k,v in orderlist do 
			local reversed = v[2];
			if ( reversed == 1 ) then reversed = -1; 
			else reversed = 1; end;
			if ( v[1] == CIM_S_TYPE ) then
				local ai = InventoryManager_GetClassIndex(typelist, a.class);
				local bi = InventoryManager_GetClassIndex(typelist, b.class);
				--Print("Comparing by type "..a.class.."("..ai..")".." & "..b.class.."("..bi..")");
				if ( ai < bi ) then
					return -1 * reversed;
				else 
					if ( ai > bi ) then 
						return 1 * reversed;
					end
				end
			end
			if ( v[1] == CIM_S_ALPHA ) then
				--DebugPrint("Comparing by name "..a.name.." & "..b.name, IM_DEBUG );
				if ( a.name > b.name ) then 
					return 1 * reversed;
				else 
					if ( a.name < b.name ) then 
						return -1 * reversed;
					end
				end
			end
			if ( v[1] == CIM_S_QUALITY ) then
				--DebugPrint("Comparing by quality "..a.quality.." & "..b.quality, IM_DEBUG );				
				if ( a.quality == 0 and b.quality > 0 ) then
					return -1 * reversed;
				end
				if ( a.quality > 0 and b.quality == 0 ) then 
					return 1 * reversed;
				end
				if ( a.quality > b.quality ) then
					return 1 * reversed;
				end
				if ( a.quality < b.quality ) then 
					return -1 * reversed;
				end
			end
			if ( v[1] == CIM_S_QUESTITEM ) then
				--DebugPrint("Comparing by QuestItem status.", IM_DEBUG);
				
				if  ( a.questItem == b.questItem ) then
				else 
					if ( a.questItem and not (b.questItem) ) then
						return 1 * reversed;
					else 
						return -1 * reversed;
					end
				end
			end
			if ( v[1] == CIM_S_UNIQUE ) then
				--DebugPrint("Comparing by uniqueness", IM_DEBUG);
				if  ( a.unique == b.unique ) then
				else 
					if ( a.unique and not (b.unique) ) then
						return 1 * reversed;
					else 
						return -1 * reversed;
					end
				end
			end
			if ( v[1] == CIM_S_SOULBOUND ) then
				--DebugPrint("Comparing by soulbound state.", IM_DEBUG);
				if  ( a.questItem == b.questItem ) then
				else 
					if ( a.questItem and not (b.questItem) ) then
						return 1 * reversed;
					else 
						return -1 * reversed;
					end
				end				
			end
			if ( v[1] == CIM_S_ITEMCOUNT ) then
				--DebugPrint("Comparing by item count.", IM_DEBUG);
				if  ( a.count == b.count ) then
				else 
					if ( a.count > b.count ) then 
						return 1 * reversed;
					else
						return -1 * reversed;
					end
				end				
			end
		end
		return 0;
	end
end

InventoryManager_DefaultCompare = CompareUsingOrderList( {{CIM_S_TYPE, CIM_NORMAL}, {CIM_S_ALPHA, CIM_NORMAL}, {CIM_S_QUALITY, CIM_NORMAL}, {CIM_S_COUNT, CIM_NORMAL}}, {{CIM_LEFTOVER, CIM_NORMAL}} );

-- Return the texture associated with the typebar or nil
function InventoryManager_GetInventoryTypeTexture(id) 
	DebugPrint("Getting texture for id("..id..") in "..CIM_BAR_TYPE, IM_DEBUG_GUI);
	
	if ( CIM_BARTYPE == CIM_BAR_TYPE ) then
		for k,v in CIM_OPTIONS_TYPE do
			if ( v.id == id ) then 
				return v.texture;
			end
		end
	elseif ( CIM_BARTYPE == CIM_BAR_SORT ) then
		for k,v in CIM_OPTIONS_SORTING do
			if ( v.id == id ) then 
				return v.texture;
			end
		end		
	end
	
end

-- Return the text associated with the typebar or nil
function InventoryManager_GetInventoryTypeText(id) 
	DebugPrint("Getting text for id("..id..") in "..CIM_BAR_TYPE, IM_DEBUG_GUI);

	if ( CIM_BARTYPE == CIM_BAR_TYPE ) then
		for k,v in CIM_OPTIONS_TYPE do
			if ( v.id == id ) then 
				return v.text;
			end
		end
	elseif ( CIM_BARTYPE == CIM_BAR_SORT ) then
		for k,v in CIM_OPTIONS_SORTING do
			if ( v.id == id ) then 
				return v.text;
			end
		end		
	end
end

-- Return the text associated with the typebar or nil
function InventoryManager_GetInventoryTypeInfo(id) 
	DebugPrint("Getting all info for id("..id..") in "..CIM_BAR_TYPE, IM_DEBUG_GUI);
	
	if ( CIM_BARTYPE == CIM_BAR_TYPE ) then
		for k,v in CIM_OPTIONS_TYPE do
			if ( v.id == id ) then 
				return v, k;
			end
		end
	elseif ( CIM_BARTYPE == CIM_BAR_SORT ) then
		for k,v in CIM_OPTIONS_SORTING do
			if ( v.id == id ) then 
				return v, k;
			end
		end
	end
	
end

-- Return the text associated or nil
function InventoryManager_GetInventoryIndexTexture(bag, slot)
	DebugPrint("Getting texture for "..bag..","..slot, IM_DEBUG_GUI);	

	if ( CIM_BARTYPE == CIM_BAR_TYPE ) then
		if ( InventoryManager_BagTypes[bag] ) then
			local k = InventoryManager_BagTypes[bag][slot];
			return CIM_OPTIONS_TYPE[k[1]].texture;
		else 
			DebugPrint("im error: "..bag, IM_DEBUG);
		end		
	elseif ( CIM_BARTYPE == CIM_BAR_SORT) then
		if ( InventoryManager_BagSortOrder[bag] ) then
			local k = InventoryManager_BagSortOrder[bag][slot];
			return CIM_OPTIONS_SORTING[k[1]].texture;
		else 
			DebugPrint("im s error: "..bag, IM_DEBUG);
		end
	end
end


-- Return the text associated or nil
function InventoryManager_GetInventoryIndexText(bag, slot)
	DebugPrint("Getting text label for "..bag..","..slot, IM_DEBUG_GUI);

	if ( CIM_BARTYPE == CIM_BAR_TYPE ) then
			local k = InventoryManager_BagTypes[bag][slot];
			if ( k ) then 
				if ( CIM_OPTIONS_TYPE[k] ) then
					return CIM_OPTIONS_TYPE[k[1]].text;
				end
			end
	elseif ( CIM_BARTYPE == CIM_BAR_SORT) then
			local k = InventoryManager_BagSortOrder[bag][slot];
			if ( k ) then 
				if ( CIM_OPTIONS_SORTING[k[1]] ) then
					return CIM_OPTIONS_SORTING[k[1]].text;
				end
			end
	end
end

-- Get a whole sort list
function InventoryManager_GetInventoryIndexInfo(bag, slot)
	DebugPrint("Getting all data for "..bag..","..slot, IM_DEBUG_GUI);

	if ( CIM_BARTYPE == CIM_BAR_TYPE ) then
		local k = InventoryManager_BagTypes[bag][slot];
		if ( k ) then 
			return CIM_OPTIONS_TYPE[k[1]];
		end
	elseif ( CIM_BARTYPE == CIM_BAR_SORT) then
		local k = InventoryManager_BagSortOrder[bag][slot];
		if ( k ) then 
			return CIM_OPTIONS_SORTING[k[1]];
		end
	end
end

-- Return the reversed state of the button
function InventoryManager_GetInventoryIndexReversed(bag, slot)
	DebugPrint("Getting reversed state for "..bag..","..slot, IM_DEBUG_GUI);

	if ( CIM_BARTYPE == CIM_BAR_TYPE ) then
		return false;
		--[[ Maybe someday I'll add reversibility to type
		if ( InventoryManager_BagTypes[bag] ) then
			if ( InventoryManager_BagTypes[bag][slot][2] == CIM_REVERSE ) then 
				return true;
			end
		end
		]]
	elseif ( CIM_BARTYPE == CIM_BAR_SORT) then
		if ( InventoryManager_BagSortOrder[bag][slot] ) then 
			if ( InventoryManager_BagSortOrder[bag][slot][2] == CIM_REVERSE ) then 
				return true;
			end
		end
	end

	return false;
end

-- Offset the bar
function InventoryManager_OffsetBar(offset) 
	CIM_BAROFFSET = CIM_BAROFFSET + offset;

	--[[
	if ( CIM_BAROFFSET >  CIM_BARCOUNTS[CIM_BARTYPE] ) then
		CIM_BAROFFSET = math.mod(CIM_BAROFFSET, CIM_BARCOUNTS[CIM_BARTYPE]);
	end
	]]
end

-- Data Setting Functions
function InventoryManager_ToggleReverse(bag, slot)
	DebugPrint("Toggling reversal for "..bag..","..slot, IM_DEBUG_GUI);

	if ( CIM_BARTYPE == CIM_BAR_TYPE ) then
		local k = InventoryManager_BagTypes[bag][slot];
		if ( k ) then 
			if ( k[2] ) then 
				if ( k[2] == CIM_NORMAL ) then 
					k[2] = CIM_REVERSE;
				else 
					k[2] = CIM_NORMAL;
				end
			end
		end
	elseif ( CIM_BARTYPE == CIM_BAR_SORT) then
		local k = InventoryManager_BagSortOrder[bag][slot];
		if ( k ) then 
			if ( k[2] == CIM_NORMAL ) then 
				k[2] = CIM_REVERSE;
			else 
				k[2] = CIM_NORMAL;
			end
		end
	end		
end

-- Set the value of a slot
function InventoryManager_SetSlot(bag, slot, value) 
	DebugPrint("Set the slot for "..bag..","..slot.." to "..value[1]..","..value[2], IM_DEBUG_GUI);

	if ( CIM_BARTYPE == CIM_BAR_TYPE ) then
		if ( value[3] == CIM_BARTYPE) then 
			if(  InventoryManager_BagTypes[bag][slot] ) then 
			if ( InventoryManager_BagTypes[bag][slot][1] ~= CIM_NONE ) then 
				if ( value[4] ) then 
					-- Swap values
					InventoryManager_BagTypes[value[4]][value[5]] = InventoryManager_BagTypes[bag][slot];
					InventoryManager_BagTypes[bag][slot] = { CIM_NONE, CIM_NORMAL };
				end
			end
			InventoryManager_BagTypes[bag][slot][1] = value[1];
			InventoryManager_BagTypes[bag][slot][2] = value[2];
			end
		end
	elseif ( CIM_BARTYPE == CIM_BAR_SORT) then
		if ( value[3] == CIM_BARTYPE) then 
			if ( InventoryManager_BagSortOrder[bag][slot][1] ~= CIM_NONE ) then 
				if ( value[4] ) then 
					-- Swap values
					InventoryManager_BagSortOrder[value[4]][value[5]] = InventoryManager_BagSortOrder[bag][slot];
					InventoryManager_BagSortOrder[bag][slot] = { CIM_NONE, CIM_NORMAL };
				end
			end
			InventoryManager_BagSortOrder[bag][slot][1] = value[1];
			InventoryManager_BagSortOrder[bag][slot][2] = value[2];
		end
	end
end

-- Obtains the held data from a typelist
function PickupIndexSort(bag, slot)
	if ( CIM_BARTYPE == CIM_BAR_TYPE ) then
		if(InventoryManager_HeldData==nil ) then 
			InventoryManager_HeldData = {};
			InventoryManager_HeldData[1] = InventoryManager_BagTypes[bag][slot][1];
			InventoryManager_HeldData[2] = InventoryManager_BagTypes[bag][slot][2];
			InventoryManager_HeldData[3] = CIM_BARTYPE;
			InventoryManager_HeldData[4] = bag;
			InventoryManager_HeldData[5] = slot;
			InventoryManager_BagTypes[bag][slot] = {CIM_NONE, CIM_NORMAL};
			PlaySound("igAbilityIconPickup");
		else 
			ErrorPrint("You're already holding something.");
		end
	elseif ( CIM_BARTYPE == CIM_BAR_SORT) then
		if(InventoryManager_HeldData==nil ) then 
			InventoryManager_HeldData = {};
			InventoryManager_HeldData[1] = InventoryManager_BagSortOrder[bag][slot][1];
			InventoryManager_HeldData[2] = InventoryManager_BagSortOrder[bag][slot][2];
			InventoryManager_HeldData[3] = CIM_BARTYPE;
			InventoryManager_HeldData[4] = bag;
			InventoryManager_HeldData[5] = slot;
			InventoryManager_BagSortOrder[bag][slot] = {CIM_NONE, CIM_NORMAL};
			PlaySound("igAbilityIconPickup");
		else 
			ErrorPrint("You're already holding something.");
		end
	end	
	
end

-- Obtains the held data value from a type list
function PickupTypeSort(id)
	local data, key = InventoryManager_GetInventoryTypeInfo(id);
	
	if ( CIM_BARTYPE == CIM_BAR_TYPE ) then
		if(InventoryManager_HeldData==nil ) then 
			InventoryManager_HeldData = {};
			InventoryManager_HeldData[1] = key;
			InventoryManager_HeldData[2] = CIM_NORMAL;
			InventoryManager_HeldData[3] = CIM_BAR_TYPE;
			DebugPrint("Data picked up: "..key.." "..CIM_NORMAL.." "..CIM_BARTYPE, IM_DEBUG_GUI);
			PlaySound("igAbilityIconPickup");
		else 
			ErrorPrint("You're already holding something.");
		end
	elseif ( CIM_BARTYPE == CIM_BAR_SORT) then
		if(InventoryManager_HeldData==nil ) then 
			InventoryManager_HeldData = {};
			InventoryManager_HeldData[1] = key;
			InventoryManager_HeldData[2] = CIM_NORMAL;
			InventoryManager_HeldData[3] = CIM_BAR_SORT;
			DebugPrint("Data picked up: "..key.." "..CIM_NORMAL.." "..CIM_BARTYPE, IM_DEBUG_GUI);
			PlaySound("igAbilityIconPickup");
		else 
			ErrorPrint("You're already holding something.");
		end
	end	
	
end

function PlaceSort(bag, slot) 
	if ( InventoryManager_HeldData ) then 
		if ( CIM_BARTYPE == InventoryManager_HeldData[3] ) then 
			DebugPrint("Held data set", IM_DEBUG_GUI);
			InventoryManager_SetSlot(bag, slot, InventoryManager_HeldData);			
		end
		DebugPrint("Held data nilled", IM_DEBUG_GUI);
		PlaySound("igAbilityIconDrop");	
		InventoryManager_HeldData = nil;		
	end
	InventoryManagerTypeRefresh();
	InventoryManagerIndexRefresh();
end


InventoryManager_Saved_ContainerFrame_OnShow = nil;
InventoryManager_Saved_ContainerFrame_OnHide = nil;

function InventoryManager_HookContainerFrameInteraction()
	if ( ContainerFrame_OnHide ~= InventoryManager_ContainerFrame_OnHide ) then
		InventoryManager_Saved_ContainerFrame_OnHide = ContainerFrame_OnHide;
		ContainerFrame_OnHide = InventoryManager_ContainerFrame_OnHide;
	end

	if ( ContainerFrame_OnShow ~= InventoryManager_ContainerFrame_OnShow ) then
		InventoryManager_Saved_ContainerFrame_OnShow = ContainerFrame_OnShow;
		ContainerFrame_OnShow = InventoryManager_ContainerFrame_OnShow;
	end
end

function InventoryManager_FixBackPackButtons()
	if ( InventoryManagerBackPackButtonsFrame ) then
		for i=1, NUM_CONTAINER_FRAMES, 1 do
			local frame = getglobal("ContainerFrame"..i);
			if ( ( frame ) and ( frame:GetID() == 0 ) ) then
				if ( InventoryManager_Enabled == 1 ) then 
					InventoryManager_AddInventoryManagerButtonsToFrame(frame:GetName());
				else
					if ( ( InventoryManagerBackPackButtonsFrame ) and ( InventoryManagerBackPackButtonsFrame:IsVisible() ) ) then
						InventoryManagerBackPackButtonsFrame:Hide();
					end
				end
				return;
			end
		end	
	end
end

function InventoryManager_ContainerFrame_OnHide()
	if ( InventoryManager_Saved_ContainerFrame_OnHide ) then
		InventoryManager_Saved_ContainerFrame_OnHide();
	end
	if ( this:GetID() == 0 ) then
		if ( ( InventoryManagerBackPackButtonsFrame ) and ( InventoryManagerBackPackButtonsFrame:IsVisible() ) ) then
			InventoryManagerBackPackButtonsFrame:Hide();
		end
	end
end

function InventoryManager_ContainerFrame_OnShow()
	if ( InventoryManager_Saved_ContainerFrame_OnShow ) then
		InventoryManager_Saved_ContainerFrame_OnShow();
	end
	if ( this:GetID() == 0 ) then
		if ( InventoryManagerBackPackButtonsFrame ) then
			if ( InventoryManager_Enabled == 1 ) then 
				InventoryManager_AddInventoryManagerButtonsToFrame(this:GetName());
				InventoryManagerBackPackButtonsFrame:Show();
			else
				InventoryManagerBackPackButtonsFrame:Hide();
			end
		end
	end
end

InventoryManager_ButtonStartX = 48;
InventoryManager_ButtonStartY = -28;

-- /script InventoryManagerBackPackButtonsFrame:ClearAllPoints(); InventoryManagerBackPackButtonsFrame:SetPoint("TOPLEFT", "ContainerFrame1", "TOPLEFT", 50, -28);
-- /script InventoryManagerBackPackButtonsFrameInventoryManagerButton:ClearAllPoints(); InventoryManagerBackPackButtonsFrameInventoryManagerButton:SetPoint("TOPLEFT", "InventoryManagerBackPackButtonsFrameSortButton", "TOPLEFT", 40, 0);
-- /script InventoryManagerBackPackButtonsFrameInventoryManagerButton:SetSize(100, 21);

function InventoryManager_AddInventoryManagerButtonsToFrame(frameName)
	local myFrameName = "InventoryManagerBackPackButtonsFrame";
	local myFrame = getglobal(myFrameName);
	if ( myFrame ) then
		myFrame:ClearAllPoints();
		myFrame:SetPoint("TOPLEFT", frameName, "TOPLEFT", InventoryManager_ButtonStartX, InventoryManager_ButtonStartY);
	end
	local buttonName = "InventoryManagerBackPackButtonsFrameSortButton";
	local button = getglobal(buttonName);
	if ( button ) then
		button:ClearAllPoints();
		button:SetPoint("TOPLEFT", myFrameName, "TOPLEFT", 0, 0);
		button:Show();
	end
	buttonName = "InventoryManagerBackPackButtonsFrameInventoryManagerButton";
	button = getglobal(buttonName);
	if ( button ) then
		button:ClearAllPoints();
		button:SetPoint("TOPLEFT", "InventoryManagerBackPackButtonsFrameSortButton", "TOPLEFT", 45, 0);
		button:Show();
	end
end
