--[[
--  This is a file for the InventoryManagerButton
--  Functions to be changed and abused.
--
--]]

function InventoryManagerButton_OnLoad()
	InventoryManager_Reset();
	InventoryManagerButton_Update(this);
	this:RegisterForDrag("LeftButton", "RightButton");
	this:RegisterForClicks("LeftButtonUp", "RightButtonUp");	
end

function InventoryManagerButton_Update(button)	
	--button:SetButtonState("NORMAL");
	
	local icon = getglobal(button:GetName().."Icon");
	local subtext = getglobal(button:GetName().."Subtext");
	local hotkey = getglobal(button:GetName().."HotKey");
	local bag, slot = InventoryManagerButton_GetButtonID(button, false);
	local iteminfo = InventoryManager_GetInventoryIndexInfo(bag, slot);
	local texture = nil;
	if ( iteminfo ~= nil ) then 
		texture = iteminfo.texture;
	end

	if ( texture ) then
		icon:SetTexture(texture);
		icon:Show();
		subtext:SetText(iteminfo.label);		
		button:SetNormalTexture("Interface\\Buttons\\UI-Quickslot2");
	else
		icon:Hide();
		subtext:SetText("");
		button:SetNormalTexture("Interface\\Buttons\\UI-Quickslot");
	end	
	if ( InventoryManager_GetInventoryIndexReversed(bag, slot) ) then
		hotkey:SetText("R");
	else
		hotkey:SetText("");		
	end
end
function InventoryManagerButton_OnUpdate()
	if ( not this.updateTooltip ) then
		return;
	end

	this.updateTooltip = this.updateTooltip - elapsed;
	if ( this.updateTooltip > 0 ) then
		return;
	end

	if ( CosmosTooltip:IsOwned(this) ) then
		InventoryManagerButton_SetTooltip();
	else
		this.updateTooltip = nil;
	end	
end

function InventoryManagerButton_OnEvent(event, message)
end

function InventoryManagerButton_OnClick(mousebutton)	
	if ( mousebutton == "RightButton" ) then 
		local bag, slot = InventoryManagerButton_GetButtonID(this, false);		
		InventoryManager_ToggleReverse(bag,slot);
	else 
		local bag, slot = InventoryManagerButton_GetButtonID(this, false);
		if ( InventoryManager_HeldData == nil ) then 
			PickupIndexSort( bag, slot);
		else 
			PlaceSort(bag, slot);
		end
	end
end

function InventoryManagerTypeButton_OnClick(button)	
	if ( mousebutton == "RightButton" ) then 
		local bag, slot = InventoryManagerButton_GetButtonID(this, false);		
		InventoryManager_ToggleReverse(bag,slot);
	else 
		-- Wipe out the old button
		InventoryManager_HeldData = nil;
		PickupTypeSort(InventoryManagerButton_GetButtonID(this, true));		
	end
end

function InventoryManagerButton_UpdateState()
	InventoryManagerButton_Update(this);
end


function InventoryManagerTypeButton_OnLoad()
	InventoryManager_Reset();
	InventoryManagerTypeButton_Update(this);
	this:RegisterForDrag("LeftButton", "RightButton");
	this:RegisterForClicks("LeftButtonUp", "RightButtonUp");		
end

function InventoryManagerTypeButton_Update(button)
	--button:SetButtonState("NORMAL");
	local icon = getglobal(button:GetName().."Icon");
	local subtext = getglobal(button:GetName().."HotKey");
	local iteminfo = InventoryManager_GetInventoryTypeInfo(InventoryManagerButton_GetButtonID(button, true));
	local texture = nil;
	if ( iteminfo ~= nil ) then 
		texture = iteminfo.texture;
	end

	if ( texture ) then
		icon:SetTexture(texture);
		icon:Show();
		subtext:SetText(iteminfo.label);
		button:SetNormalTexture("Interface\\Buttons\\UI-Quickslot2");
	else
		icon:Hide();
		subtext:SetText("");		
		button:SetNormalTexture("Interface\\Buttons\\UI-Quickslot");
	end	

	if ( InventoryManager_HeldData ~= nil ) then
		local d, k = InventoryManager_GetInventoryTypeInfo(button:GetID());
		if ( InventoryManager_HeldData[1] == k ) then
			button:SetChecked(1);
		else
			button:SetChecked(0);
			
		end	
	else
		button:SetChecked(0);		
	end	
end

-- Update the tooltip if needed
function InventoryManagerTypeButton_OnUpdate(elapsed)
	if ( not this.updateTooltip ) then
		return;
	end

	this.updateTooltip = this.updateTooltip - elapsed;
	if ( this.updateTooltip > 0 ) then
		return;
	end

	if ( CosmosTooltip:IsOwned(this) ) then
		InventoryManagerTypeButton_SetTooltip();
	else
		this.updateTooltip = nil;
	end
end

function InventoryManagerTypeButton_OnEvent(event, message)
end

function InventoryManagerTypeButton_UpdateState()
	InventoryManagerTypeButton_Update(this);
end

-- Obtains the ID of the actual reference, rather than the button itself
function InventoryManagerButton_GetButtonID(button, typed)
	if( button == nil ) then
		message("nil button passed into GetButtonID(), contact Alex");
		return 0;
	end
	if ( typed ) then
		local id = (button:GetID() + CIM_BAROFFSET);
	
		local i=0;
		while ( id > CIM_BARCOUNTS[CIM_BARTYPE] ) do
			id = id - CIM_BARCOUNTS[CIM_BARTYPE];
			i = i+1;
			if ( i > 100 ) then break; end
		end	
		while ( id < 0 ) do
			id = id + CIM_BARCOUNTS[CIM_BARTYPE];
			i = i+1;
			if ( i > 100 ) then break; end
		end
		return id;
	else
		local id = (button:GetID());
		
		local buttonidbag = (id-math.mod(id,10))/10;
		local bag = CIM_BAGINDEXES[CIM_BUTTONIDTOBAGNAME[buttonidbag]];
		
DebugPrint(button:GetName().." is ".." Bag "..bag, IM_DEBUG_GUI);
		
		-- Return Bag, Index if its not a type id
		return bag, math.mod(id,10);
	end
end

-- Sets the tooltip
function InventoryManagerTypeButton_SetTooltip()
	CosmosTooltip:SetOwner(this, "ANCHOR_RIGHT");
	local text = InventoryManager_GetInventoryTypeText(InventoryManagerButton_GetButtonID(this, true));
	if ( text ) then 
		CosmosTooltip:SetText(text);
		CosmosTooltip:Show();
	end
	this.updateTooltip = nil;						
end

-- Sets the tooltip
function InventoryManagerButton_SetTooltip()
	CosmosTooltip:SetOwner(this, "ANCHOR_RIGHT");
	local bag, slot = InventoryManagerButton_GetButtonID(this, false);
	local text = InventoryManager_GetInventoryIndexText(bag, slot);
	if ( text ) then 
		CosmosTooltip:SetText(text);
		CosmosTooltip:Show();
	end
	this.updateTooltip = nil;
end

-- Hides the tooltip
function InventoryManagerButton_HideTooltip()
	CosmosTooltip:Hide();
	this.updateTooltip = nil;
end

-- Hides the tooltip
function InventoryManagerTypeButton_HideTooltip()
	CosmosTooltip:Hide();
	this.updateTooltip = nil;
end

-- Refreshes the Type list
function InventoryManagerTypeRefresh()
	for i=1, 7, 1 do
		button = getglobal("InventoryManagerTypeBarButton"..i);
		if ( button ) then
			button:SetButtonState("NORMAL");
			button:SetChecked(0);
			InventoryManagerTypeButton_Update(button);
		end
	end
end

-- Refreshes the Bag list vew
function InventoryManagerIndexRefresh()
	for i=1, 5, 1 do
		for j=1, 6, 1 do
			button = getglobal("InventoryManagerBag"..i.."BarButton"..j);
			if ( button ) then
				button:SetButtonState("NORMAL");
				button:SetChecked(0);
				InventoryManagerButton_Update(button);
			end
		end
	end
end

-- Handle onclicks
function InventoryManagerFrameTab_OnClick() 
	CIM_BARTYPE = CIM_BARLIST[this:GetID()];
	InventoryManagerTypeRefresh();
	InventoryManagerIndexRefresh();
end

